#!/usr/bin/env python

#****************************************************************************************************************************************************
# Copyright (c) 2014 Freescale Semiconductor, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright notice,
#      this list of conditions and the following disclaimer.
#
#    * Redistributions in binary form must reproduce the above copyright notice,
#      this list of conditions and the following disclaimer in the documentation
#      and/or other materials provided with the distribution.
#
#    * Neither the name of the Freescale Semiconductor, Inc. nor the names of
#      its contributors may be used to endorse or promote products derived from
#      this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#****************************************************************************************************************************************************

#import cProfile
#import pstats
import argparse
import sys
from FslBuildGen import IOUtil, PluginConfig
from FslBuildGen import Util
from FslBuildGen.Config import Config
from FslBuildGen.Main import *

__g_verbosityLevel = 0
__g_debugEnabled = False
__g_allowDevelopmentPlugins = False

def GetTitle():
    return 'FslBuildGen V0.7.0 alpha'


def ShowTitleIfNecessary():
    global __g_verbosityLevel
    if __g_verbosityLevel > 0:
        print(GetTitle())


def TestableMain(args, currentDir):
    typeSdk = 'sdk'

    toolConfigPath = IOUtil.GetExecutablePath()
    toolConfigPath = IOUtil.Join(toolConfigPath, "FslBuildGen.xml")

    toolConfig = LoadToolConfig(toolConfigPath)
    allowDevelopmentPlugins = True if args.dev else False
    config = Config(toolConfig, args.verbosity, args.type, allowDevelopmentPlugins)

    if args.DryRun:
        config.DisableWrite = True


    config.LogPrint(GetTitle())

    if args.graph:
        PluginConfig.EnableGraph();

    theFiles = []
    if not config.IsSDKBuild:
        theFile = IOUtil.Join(currentDir, "Fsl.gen")
        if not os.path.isfile(theFile):
            raise Exception("File not found: '%s'" % (theFile))
        theFiles.append(theFile)

    DoGenerateBuildFiles(config, theFiles, args.platform)


def AddDefaultOptions(parser):
    parser.add_argument('-v', '--verbosity', action='count', default=0, help='Set verbosity level')
    parser.add_argument('--debug', action='store_true',  help='Enable script debugging')
    parser.add_argument('--dev', action='store_true',  help='Allow plugins in development')


def EarlyArgumentParser():
    global __g_verbosityLevel
    global __g_debugEnabled
    global __g_allowDevelopmentPlugins
    ### Parse the initial options this allows us to use the required debug and verbosity levels while 
    ### creating the actual command line argumnets.
    try:
        parser = argparse.ArgumentParser(add_help=False)
        AddDefaultOptions(parser)
        args, unknown = parser.parse_known_args()
        __g_verbosityLevel = args.verbosity
        __g_debugEnabled = True if args.debug else False;
        __g_allowDevelopmentPlugins = True if args.dev else False;
    except (Exception) as ex:
        print("ERROR: %s" % ex.message)
        if __g_debugEnabled:
            raise
        else:
            return False
    return True


def Main():
    global __g_verbosityLevel
    global __g_debugEnabled
    global __g_allowDevelopmentPlugins

    if not EarlyArgumentParser():
        return

    generatorPlugins = PluginConfig.GetGeneratorPlugins(__g_allowDevelopmentPlugins)
    generatorIds = [entry.Id for entry in generatorPlugins]
                

    ### Do the actual command line parsing
    parser = argparse.ArgumentParser(description='Create the build files the various supported platforms.')
    AddDefaultOptions(parser)
    parser.add_argument('-t', '--type', default=PluginConfig.TYPE_DEFAULT,  help='Select generator type: %s, sdk' % (PluginConfig.TYPE_DEFAULT))
    parser.add_argument('-p', '--platform', default=PluginConfig.PLATFORM_ID_ALL,  help='Select build files platform: %s' % (", ".join(generatorIds)))
    parser.add_argument('--graph', action='store_true',  help='Generate a dependency graph using dot (requires the graphviz dot executable in path)')
    parser.add_argument('--DryRun', action='store_true',  help='No files will be created')

    currentDir = Util.EnsureUTF8(os.getcwd())
    try:
        args = parser.parse_args()
        TestableMain(args, currentDir)
    except GroupedException as ex:
        ShowTitleIfNecessary()
        for entry in ex.ExceptionList:
            print("ERROR: %s" % entry.message)
        if __g_debugEnabled:
            raise
    except (Exception) as ex:
        print("ERROR: %s" % ex.message)
        if __g_debugEnabled:
            raise

Main()
#cProfile.run('Main()', 'restats')
#p = pstats.Stats('restats')
#p.strip_dirs().sort_stats(-1).print_stats()
#p.sort_stats('cumulative').print_stats(10)
#p.sort_stats('time').print_stats(10)
#p.sort_stats('tottime').print_stats(100)
