#!/usr/bin/env python

#****************************************************************************************************************************************************
# Copyright (c) 2014 Freescale Semiconductor, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright notice,
#      this list of conditions and the following disclaimer.
#
#    * Redistributions in binary form must reproduce the above copyright notice,
#      this list of conditions and the following disclaimer in the documentation
#      and/or other materials provided with the distribution.
#
#    * Neither the name of the Freescale Semiconductor, Inc. nor the names of
#      its contributors may be used to endorse or promote products derived from
#      this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#****************************************************************************************************************************************************

from FslBuildGen.Exceptions import *
from FslBuildGen import PackageConfig
from FslBuildGen.PackageResolver import PackageResolver
from FslBuildGen.Generator import GenerateBuildScript
from FslBuildGen.Generator.GeneratorAndroid import GeneratorAndroid
from FslBuildGen.Generator.GeneratorGNUmakefile import GeneratorGNUmakefile
from FslBuildGen.Generator.GeneratorVC import GeneratorVC
from FslBuildGen.Generator.GeneratorDot import GeneratorDot

PLATFORM_ID_ALL = "all"
TYPE_DEFAULT = "default"
TYPE_UNIT_TEST = "UnitTest"


class GeneratorPlugin(object):
    def __init__(self, name):
        super(GeneratorPlugin, self).__init__()
        self.Name = name;
        self.Id = name.lower()
        self.InDevelopment = False
        self.DotEnabled = False

    def Generate(self, config, packageLoader):
        pass

    def GenerateDone(self, config, packages, name):
        if self.DotEnabled:
            GeneratorDot(config, packages, name)
        return packages

class GeneratorPluginAndroid(GeneratorPlugin):
    def __init__(self):
        super(GeneratorPluginAndroid, self).__init__(PackageConfig.PLATFORM_ANDROID)

    def Generate(self, config, packageLoader):
        packageResolver = PackageResolver(config, self.Name, packageLoader.GenFiles, False)
        generator = GeneratorAndroid(config, packageResolver.Packages, self.Name)
        return self.GenerateDone(config, packageResolver.Packages, self.Name)


class GeneratorPluginUbuntu(GeneratorPlugin):
    def __init__(self):
        super(GeneratorPluginUbuntu, self).__init__(PackageConfig.PLATFORM_UBUNTU)

    def Generate(self, config, packageLoader):
        packageResolver = PackageResolver(config, self.Name, packageLoader.GenFiles, False)
        generator = GeneratorGNUmakefile(config, packageResolver.Packages, "GNUmakefile", "GNUmakefile_exe", "GNUmakefile_lib")
        GenerateBuildScript.GenerateBuildScriptAll(config, packageResolver.Packages)
        return self.GenerateDone(config, packageResolver.Packages, self.Name)


class GeneratorPluginYocto(GeneratorPlugin):
    def __init__(self):
        super(GeneratorPluginYocto, self).__init__(PackageConfig.PLATFORM_YOCTO)

    def Generate(self, config, packageLoader):
        packageResolver = PackageResolver(config, self.Name, packageLoader.GenFiles, False)
        generator = GeneratorGNUmakefile(config, packageResolver.Packages, "Makefile", "Makefile_exe", "Makefile_lib")
        GenerateBuildScript.GenerateBuildScriptAll(config, packageResolver.Packages)
        return self.GenerateDone(config, packageResolver.Packages, self.Name)


class GeneratorPluginWindows(GeneratorPlugin):
    def __init__(self):
        super(GeneratorPluginWindows, self).__init__(PackageConfig.PLATFORM_WINDOWS)
        self.InDevelopment = False

    def Generate(self, config, packageLoader):
        packageResolver = PackageResolver(config, self.Name, packageLoader.GenFiles, True)
        generator = GeneratorVC(config, packageResolver.Packages, self.Name)
        return self.GenerateDone(config, packageResolver.Packages, self.Name)


__g_generatorPlugins = [GeneratorPlugin(PLATFORM_ID_ALL), GeneratorPluginAndroid(), GeneratorPluginUbuntu(), GeneratorPluginYocto(), GeneratorPluginWindows()]
__g_generatorPluginDict = {}

for entry in __g_generatorPlugins:
    __g_generatorPluginDict[entry.Id] = entry


def GetGeneratorPlugins(allowDevelopmentPlugins):
    if allowDevelopmentPlugins:
        return __g_generatorPlugins
    else:
        return [entry for entry in __g_generatorPlugins if not entry.InDevelopment]


def GetGeneratorPluginById(id):
    if not __g_generatorPluginDict.has_key(id):
        raise UsageErrorException("Unknown platform: '%s'" % (id))
    return __g_generatorPluginDict[id]

def EnableGraph():
  for entry in __g_generatorPlugins:
      entry.DotEnabled = True;

