/****************************************************************************************************************************************************
* Copyright (c) 2014 Freescale Semiconductor, Inc.
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*
*    * Redistributions of source code must retain the above copyright notice,
*      this list of conditions and the following disclaimer.
*
*    * Redistributions in binary form must reproduce the above copyright notice,
*      this list of conditions and the following disclaimer in the documentation
*      and/or other materials provided with the distribution.
*
*    * Neither the name of the Freescale Semiconductor, Inc. nor the names of
*      its contributors may be used to endorse or promote products derived from
*      this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
* ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
****************************************************************************************************************************************************/

#include <FslGraphicsEGL/Window/EGLNativeWindowSystemFactory.hpp>
#include <FslGraphicsEGL/Window/IEGLNativeWindowSystem.hpp>

#if _WIN32
#include "Win32/EGLNativeWindowWin32.hpp"
namespace Fsl
{
  typedef EGLNativeWindowSystemWin32 EGLNATIVEWINDOWSYSTEM;
}
#elif defined(__ANDROID__)
#include "Android/EGLNativeWindowAndroid.hpp"
namespace Fsl
{
  typedef EGLNativeWindowSystemAndroid EGLNATIVEWINDOWSYSTEM;
}
#elif defined(__linux__)
#ifdef EGL_USE_X11
  #include "X11/EGLNativeWindowX11.hpp"
    namespace Fsl
    {
      typedef EGLNativeWindowSystemX11 EGLNATIVEWINDOWSYSTEM;
    }
  #else
    #if ((defined(EGL_API_FB)) && !(defined(EGL_API_WL)))
      #include "FB/EGLNativeWindowFB.hpp"
      namespace Fsl
      {
        typedef EGLNativeWindowSystemFB EGLNATIVEWINDOWSYSTEM;
      }
    #else
      #ifdef EGL_API_WL
      #include "Wayland/EGLNativeWindowWayland.hpp"
      namespace Fsl
      {
        typedef EGLNativeWindowSystemWayland EGLNATIVEWINDOWSYSTEM;
      }
      #else
        #ifdef EGL_API_DFB
          #include "DirectFB/EGLNativeWindowDFB.hpp"
          namespace Fsl
          {
            typedef EGLNativeWindowSystemDFB EGLNATIVEWINDOWSYSTEM;
          }
        #endif //EGL_API_DFB
      #endif //EGL_API_WL
    #endif //EGL_API_FB
  #endif //EGL_USE_X11
#endif //_WIN32

namespace Fsl
{

  const std::shared_ptr<IEGLNativeWindowSystem> EGLNativeWindowSystemFactory::Allocate(const NativeWindowSystemSetup& setup)
  {
    return std::shared_ptr<IEGLNativeWindowSystem>(new EGLNATIVEWINDOWSYSTEM(setup));
  }

}
