#if ! defined(__ANDROID__) && defined(__linux__) &&  defined(EGL_USE_X11)// FIX: use another define
/****************************************************************************************************************************************************
* Copyright (c) 2014 Freescale Semiconductor, Inc.
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*
*    * Redistributions of source code must retain the above copyright notice,
*      this list of conditions and the following disclaimer.
*
*    * Redistributions in binary form must reproduce the above copyright notice,
*      this list of conditions and the following disclaimer in the documentation
*      and/or other materials provided with the distribution.
*
*    * Neither the name of the Freescale Semiconductor, Inc. nor the names of
*      its contributors may be used to endorse or promote products derived from
*      this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
* ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
****************************************************************************************************************************************************/

#include <iostream>
#include "EGLNativeWindowX11.hpp"
#include <FslBase/Log/Log.hpp>
#include <FslGraphicsEGL/Exceptions.hpp>
#include <FslGraphicsEGL/EGLCheck.hpp>
#include <X11/Xlib.h>
#include <X11/extensions/Xrandr.h>
#include <X11/keysym.h>
#include <FslGraphics/Window/INativeWindowEventQueue.hpp>
#include <FslGraphics/Window/NativeWindowEventHelper.hpp>

namespace Fsl
{
  namespace
  {
    // These exist because we told EGL to not contaminate other files with X11 defines (since some might collide with normal code)
    typedef Display* X11EGLNativeDisplayType;
    typedef Pixmap   X11EGLNativePixmapType;
    typedef Window   X11EGLNativeWindowType;
    std::weak_ptr<INativeWindowEventQueue> g_eventQueue;
    int WaitForMap(Display* display, XEvent* event, XPointer arg)
    {
      return (event->type == MapNotify && event->xmap.window == *reinterpret_cast<Window*>(arg));
    }

    VirtualMouseButton::Enum MouseToVirtualMouse(unsigned int button)
    {
      switch (button)
      {
          case Button1:
            return VirtualMouseButton::Left;

          case Button2:
            return VirtualMouseButton::Middle;

          case Button3:
            return VirtualMouseButton::Right;

          default:
            return VirtualMouseButton::Left;
      }
    }


    VirtualKey::Enum KeyToVirtualKey(XKeyEvent * kEvent)
      {
        KeySym keyId = XLookupKeysym(kEvent, 0);

        switch(keyId)
        {
          case XK_A:
          case XK_a:
            return VirtualKey::A;

          case XK_B:
          case XK_b:
            return VirtualKey::B;

          case XK_C:
          case XK_c:
            return VirtualKey::C;

          case XK_D:
          case XK_d:
            return VirtualKey::D;

          case XK_E:
          case XK_e:
            return VirtualKey::E;

          case XK_F:
          case XK_f:
            return VirtualKey::F;

          case XK_G:
          case XK_g:
            return VirtualKey::G;

          case XK_H:
          case XK_h:
            return VirtualKey::H;

          case XK_I:
          case XK_i:
            return VirtualKey::I;

          case XK_J:
          case XK_j:
            return VirtualKey::J;

          case XK_K:
          case XK_k:
            return VirtualKey::K;

          case XK_L:
          case XK_l:
            return VirtualKey::L;

          case XK_M:
          case XK_m:
            return VirtualKey::M;

          case XK_N:
          case XK_n:
            return VirtualKey::N;

          case XK_O:
          case XK_o:
            return VirtualKey::O;

          case XK_P:
          case XK_p:
            return VirtualKey::P;

          case XK_Q:
          case XK_q:
            return VirtualKey::Q;

          case XK_R:
          case XK_r:
            return VirtualKey::R;

          case XK_S:
          case XK_s:
            return VirtualKey::S;

          case XK_T:
          case XK_t:
            return VirtualKey::T;

          case XK_U:
          case XK_u:
            return VirtualKey::U;

          case XK_V:
          case XK_v:
            return VirtualKey::V;

          case XK_W:
          case XK_w:
            return VirtualKey::W;

          case XK_X:
          case XK_x:
            return VirtualKey::X;

          case XK_Y:
          case XK_y:
            return VirtualKey::Y;

          case XK_Z:
          case XK_z:
            return VirtualKey::Z;

          case XK_0:
            return VirtualKey::Code0;

          case XK_1:
            return VirtualKey::Code1;

          case XK_2:
            return VirtualKey::Code2;

          case XK_3:
            return VirtualKey::Code3;

          case XK_4:
            return VirtualKey::Code4;

          case XK_5:
            return VirtualKey::Code5;

          case XK_6:
            return VirtualKey::Code6;

          case XK_7:
            return VirtualKey::Code7;

          case XK_8:
            return VirtualKey::Code8;

          case XK_9:
            return VirtualKey::Code9;

          case XK_Escape:
            return VirtualKey::Escape;

          case XK_Tab:
            return VirtualKey::Tab;

          case XK_Return:
            return VirtualKey::Return;

          case XK_space:
            return VirtualKey::Space;

          case XK_BackSpace:
            return VirtualKey::Backspace;

          case XK_Up:
            return VirtualKey::UpArrow;

          case XK_Down:
            return VirtualKey::DownArrow;

          case XK_Left:
            return VirtualKey::LeftArrow;

          case XK_Right:
            return VirtualKey::RightArrow;

          case XK_F1:
            return VirtualKey::F1;

          case XK_F2:
            return VirtualKey::F2;

          case XK_F3:
            return VirtualKey::F3;

          case XK_F4:
            return VirtualKey::F4;

          case XK_F5:
            return VirtualKey::F5;

          case XK_F6:
            return VirtualKey::F6;

          case XK_F7:
            return VirtualKey::F7;

          case XK_F8:
            return VirtualKey::F8;

          case XK_F9:
            return VirtualKey::F9;

          case XK_F10:
            return VirtualKey::F10;

          case XK_F11:
            return VirtualKey::F11;

          case XK_F12:
            return VirtualKey::F12;

          case XK_Insert:
            return VirtualKey::Insert;

          case XK_Delete:
            return VirtualKey::Delete;

          case XK_Home:
            return VirtualKey::Home;

          case XK_End:
            return VirtualKey::End;

          case XK_Page_Up:
            return VirtualKey::PageUp;

          case XK_Page_Down:
            return VirtualKey::PageDown;

          case XK_Print:
            return VirtualKey::PrintScreen;

          case XK_Pause:
            return VirtualKey::ScrollLock;

          case XK_Scroll_Lock:
            return VirtualKey::ScrollLock;

          default :
            return VirtualKey::Tab;
        }
      }
  }


  EGLNativeWindowSystemX11::EGLNativeWindowSystemX11(const NativeWindowSystemSetup& setup)
  {
    FSLLOG_WARNING_IF(setup.GetConfig().GetDisplayId() != 0, "X11 only supports the main display. Using DisplayId 0 instead of " << setup.GetConfig().GetDisplayId());
    m_hDisplay = XOpenDisplay(nullptr);
    if (!m_hDisplay)
      throw EGLGraphicsException("Could not open display");
  }


  EGLNativeWindowSystemX11::~EGLNativeWindowSystemX11()
  {
    XCloseDisplay(static_cast<X11EGLNativeDisplayType>(m_hDisplay));
  }


  EGLNativeDisplayType EGLNativeWindowSystemX11::GetDisplayType() const
  {
    return m_hDisplay;
  }

  std::shared_ptr<IEGLNativeWindow> EGLNativeWindowSystemX11::CreateNativeWindow(const NativeWindowSetup& nativeWindowSetup, const NativeEGLSetup& nativeEglSetup)
  {
    const std::shared_ptr<EGLNativeWindowX11> ptr(new EGLNativeWindowX11(nativeWindowSetup, nativeEglSetup));
    m_window = ptr;
    return ptr;
  }


  bool EGLNativeWindowSystemX11::ProcessMessages(const NativeWindowProcessMessagesArgs& args)
  {
    const std::shared_ptr<EGLNativeWindowX11> window = m_window.lock();
    const X11EGLNativeDisplayType hDisplayType = static_cast<X11EGLNativeDisplayType>(m_hDisplay);
    VirtualKey::Enum keyCode;
    std::shared_ptr<INativeWindowEventQueue> eventQueue = g_eventQueue.lock();
    Point2 mousePosition;
    VirtualMouseButton::Enum mouseButton;
    bool bQuit = false;
    XEvent event;
    static int wheelEvents = 0;

    while (XPending(hDisplayType) > 0)
    {
      XNextEvent(hDisplayType, &event);

      switch(event.type)
      {
      case ConfigureNotify:
        // std::cout << "Window moved or resized" << std::endl;

        break;
      case DestroyNotify:
        bQuit = true;
        break;
      case ClientMessage:
        if( window )
        {
          if ((Atom)event.xclient.data.l[0] == window->m_wm_delete_window)
          {
            bQuit = true;
          }
        }
        break;
      case ButtonPress:
      {
        mousePosition.X = event.xbutton.x;
        mousePosition.Y = event.xbutton.y;
        if(event.xbutton.button<Button4)
        {
          //Regular Mouse Buttons
          mouseButton = MouseToVirtualMouse(event.xbutton.button);
          if( eventQueue )
            eventQueue->PostEvent(NativeWindowEventHelper::EncodeInputMouseButtonEvent(mouseButton, true, mousePosition));
        }
        else
        {
          //Wheel Event
          if(Button4==event.xbutton.button)
            wheelEvents++;
          else if(Button5==event.xbutton.button)
            wheelEvents--;
          if( eventQueue )
            eventQueue->PostEvent(NativeWindowEventHelper::EncodeInputMouseWheelEvent(wheelEvents, mousePosition));
        }
      }
      break;

      case ButtonRelease:
      {
        mousePosition.X = event.xbutton.x;
        mousePosition.Y = event.xbutton.y;
        mouseButton = MouseToVirtualMouse(event.xbutton.button);
        if( eventQueue )
          eventQueue->PostEvent(NativeWindowEventHelper::EncodeInputMouseButtonEvent(mouseButton, false, mousePosition));
      }
      break;

      case KeyPress:
      {
        keyCode = KeyToVirtualKey(&event.xkey);
        if( eventQueue )
          eventQueue->PostEvent(NativeWindowEventHelper::EncodeInputKeyEvent(keyCode, true));
      }
      break;

      case KeyRelease:
      {
        keyCode = KeyToVirtualKey(&event.xkey);
        if( eventQueue )
          eventQueue->PostEvent(NativeWindowEventHelper::EncodeInputKeyEvent(keyCode, false));
      }
      break;

      case MotionNotify:
      {
        mousePosition.X = event.xmotion.x;
        mousePosition.Y = event.xmotion.y;
        if( eventQueue )
          eventQueue->PostEvent(NativeWindowEventHelper::EncodeInputMouseMoveEvent(mousePosition));
      }
      break;

      }

    }
    return !bQuit;
  }


  EGLNativeWindowX11::EGLNativeWindowX11(const NativeWindowSetup& nativeWindowSetup, const NativeEGLSetup& nativeEglSetup)
    : m_hDisplayType(nativeEglSetup.DisplayType)
    , m_pVisual(nullptr)
  {
    const NativeWindowConfig nativeWindowConfig = nativeWindowSetup.GetConfig();
    const EGLDisplay hDisplay = nativeEglSetup.Display;
    const EGLConfig eglConfig = nativeEglSetup.Config;
    int windowWidth, windowHeight, windowX, windowY;
    g_eventQueue = nativeWindowSetup.GetEventQueue();

    const X11EGLNativeDisplayType hDisplayType = static_cast<X11EGLNativeDisplayType>(m_hDisplayType);

    // Get the native visual id that matches the egl config
    EGLint nativeVisualId;
    EGL_CHECK(eglGetConfigAttrib(hDisplay, eglConfig, EGL_NATIVE_VISUAL_ID, &nativeVisualId));

    XVisualInfo visualInfo;
    visualInfo.visualid = nativeVisualId;

    int visualItemCount;
    m_pVisual = XGetVisualInfo(hDisplayType, VisualIDMask, &visualInfo, &visualItemCount);
    if( m_pVisual == nullptr )
      throw EGLGraphicsException("Could not get native visual");

    const int screen = DefaultScreen(hDisplayType);
    const Window rootwindow = RootWindow(hDisplayType, screen);

    //GET SCREEN RESOURCES FOR THE DEVICE
    XRRScreenResources * screenResources;
    screenResources = XRRGetScreenResources(hDisplayType, rootwindow);

    //We now have info on the Screen Resources, get information about the available CRTs
    XRRCrtcInfo * crtDeviceInformation;


    crtDeviceInformation = XRRGetCrtcInfo(hDisplayType, screenResources, screenResources->crtcs[0]);
    //printf("Specific CRT information %dx%d\n",crtDeviceInformation->width, crtDeviceInformation->height);


    if (nativeWindowConfig.GetWindowMode() != WindowMode::Window)
    {
      FSLLOG("WARNING: Window Size/Position values not set, setting them to Full Resolution of Primary CRT");

      windowWidth  = crtDeviceInformation->width;
      windowHeight = crtDeviceInformation->height;

      windowX = 0;
      windowY = 0;
    }
    else
    {
      const Rectangle windowRectangle = nativeWindowConfig.GetWindowRectangle();
      windowWidth = windowRectangle.Width;
      windowHeight = windowRectangle.Height;
      windowX = windowRectangle.X;
      windowY = windowRectangle.Y;
    }


    m_colormap = XCreateColormap(hDisplayType, rootwindow, m_pVisual->visual, AllocNone);


    const unsigned long mask = CWBackPixel | CWBorderPixel | CWEventMask | CWColormap;

    XSetWindowAttributes windowAttributes;
    windowAttributes.colormap = m_colormap;
    windowAttributes.background_pixel = 0xFFFFFFFF;
    windowAttributes.border_pixel = 0;
    windowAttributes.event_mask = StructureNotifyMask | ExposureMask;


    m_hWindow = XCreateWindow(hDisplayType, rootwindow,
      windowX, windowY, windowWidth, windowHeight,
      0, m_pVisual->depth, InputOutput, m_pVisual->visual, mask, &windowAttributes);

    X11EGLNativeWindowType hWindow = static_cast<X11EGLNativeWindowType>(m_hWindow);
    XMapWindow(hDisplayType, hWindow);

    XEvent event;
    XIfEvent(hDisplayType, &event, WaitForMap, reinterpret_cast<XPointer>(&hWindow));

    XSetWMColormapWindows(hDisplayType, hWindow, &hWindow, 1);

    XFlush(hDisplayType);

    // Hook up and listen for window close
    m_wm_delete_window=XInternAtom(hDisplayType, "WM_DELETE_WINDOW", True);
    XSetWMProtocols(hDisplayType, hWindow, &m_wm_delete_window, 1);

    // Listen for various events
    const long evMask = ExposureMask | StructureNotifyMask | VisibilityChangeMask |
      ButtonPressMask | ButtonReleaseMask | KeyPressMask | KeyReleaseMask |
      EnterWindowMask | LeaveWindowMask | PointerMotionMask;
    XSelectInput(hDisplayType, hWindow, evMask);


    { // Post the activation message to let the framework know we are ready
      std::shared_ptr<INativeWindowEventQueue> eventQueue = g_eventQueue.lock();
      if( eventQueue )
        eventQueue->PostEvent(NativeWindowEventHelper::EncodeWindowActivationEvent(true));
    }
  }


  EGLNativeWindowX11::~EGLNativeWindowX11()
  {
    const X11EGLNativeDisplayType hDisplayType = static_cast<X11EGLNativeDisplayType>(m_hDisplayType);
    X11EGLNativeWindowType hWindow = static_cast<X11EGLNativeWindowType>(m_hWindow);

    // X windows clean up
    XDestroyWindow(hDisplayType, hWindow);

    XFreeColormap(hDisplayType, m_colormap);

    if( m_pVisual != nullptr )
      XFree(m_pVisual);
  }


  EGLNativeWindowType EGLNativeWindowX11::GetWindowType() const
  {
    return m_hWindow;
  }

}
#endif
